@@ -1,5 +1,6 @@
 """Tests for the decorators we've created for IPython.
 """
+from __future__ import print_function
 
 # Module imports
 # Std lib
@@ -11,7 +12,7 @@ import nose.tools as nt
 
 # Our own
 from IPython.testing import decorators as dec
-
+from IPython.testing.skipdoctest import skip_doctest
 
 #-----------------------------------------------------------------------------
 # Utilities
@@ -32,15 +33,21 @@ def getargspec(obj):
     if inspect.isfunction(obj):
         func_obj = obj
     elif inspect.ismethod(obj):
-        func_obj = obj.im_func
+        func_obj = obj.__func__
     else:
-        raise TypeError, 'arg is not a Python function'
-    args, varargs, varkw = inspect.getargs(func_obj.func_code)
-    return args, varargs, varkw, func_obj.func_defaults
+        raise TypeError('arg is not a Python function')
+    args, varargs, varkw = inspect.getargs(func_obj.__code__)
+    return args, varargs, varkw, func_obj.__defaults__
 
 #-----------------------------------------------------------------------------
 # Testing functions
 
+@dec.as_unittest
+def trivial():
+    """A trivial test"""
+    pass
+
+
 @dec.skip
 def test_deliberately_broken():
     """A deliberately broken test - we want to skip this one."""
@@ -54,16 +61,16 @@ def test_deliberately_broken2():
 
 # Verify that we can correctly skip the doctest for a function at will, but
 # that the docstring itself is NOT destroyed by the decorator.
-@dec.skip_doctest
+@skip_doctest
 def doctest_bad(x,y=1,**k):
     """A function whose doctest we need to skip.
 
     >>> 1+1
     3
     """
-    print 'x:',x
-    print 'y:',y
-    print 'k:',k
+    print('x:',x)
+    print('y:',y)
+    print('k:',k)
 
 
 def call_doctest_bad():
@@ -89,11 +96,12 @@ def test_skip_dt_decorator():
     # Fetch the docstring from doctest_bad after decoration.
     val = doctest_bad.__doc__
     
-    assert check==val,"doctest_bad docstrings don't match"
+    nt.assert_equal(check,val,"doctest_bad docstrings don't match")
+
 
 # Doctest skipping should work for class methods too
-class foo(object):
-    """Foo
+class FooClass(object):
+    """FooClass
 
     Example:
 
@@ -101,24 +109,24 @@ class foo(object):
     2
     """
 
-    @dec.skip_doctest
+    @skip_doctest
     def __init__(self,x):
-        """Make a foo.
+        """Make a FooClass.
 
         Example:
 
-        >>> f = foo(3)
+        >>> f = FooClass(3)
         junk
         """
-        print 'Making a foo.'
+        print('Making a FooClass.')
         self.x = x
         
-    @dec.skip_doctest
+    @skip_doctest
     def bar(self,y):
         """Example:
 
-        >>> f = foo(3)
-        >>> f.bar(0)
+        >>> ff = FooClass(3)
+        >>> ff.bar(0)
         boom!
         >>> 1/0
         bam!
@@ -128,15 +136,14 @@ class foo(object):
     def baz(self,y):
         """Example:
 
-        >>> f = foo(3)
-        Making a foo.
-        >>> f.baz(3)
+        >>> ff2 = FooClass(3)
+        Making a FooClass.
+        >>> ff2.baz(3)
         True
         """
         return self.x==y
 
 
-
 def test_skip_dt_decorator2():
     """Doctest-skipping decorator should preserve function signature.
     """
@@ -150,12 +157,13 @@ def test_skip_dt_decorator2():
 
 @dec.skip_linux
 def test_linux():
-    nt.assert_not_equals(sys.platform,'linux2',"This test can't run under linux")
+    nt.assert_false(sys.platform.startswith('linux'),"This test can't run under linux")
 
 @dec.skip_win32
 def test_win32():
-    nt.assert_not_equals(sys.platform,'win32',"This test can't run under windows")
+    nt.assert_not_equal(sys.platform,'win32',"This test can't run under windows")
 
 @dec.skip_osx
 def test_osx():
-    nt.assert_not_equals(sys.platform,'darwin',"This test can't run under osx")
+    nt.assert_not_equal(sys.platform,'darwin',"This test can't run under osx")
+
